// outline.frag

#version 130

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int lightNumber;
uniform int lightRecieve[7];
uniform int useTexAlpha;
uniform int dissolveMode;

uniform float projInv11;
uniform float projInv15;

uniform int shadowVolumeMode;
uniform float edgeThreshold;
uniform int bFlat;

uniform sampler2D SM_Texture;
uniform mat4 SM_LightViewMatrix;
//uniform mat4 SM_LightProjMatrix; //KvȂB
uniform float SM_OrthoSize; //OrthoScaleKv
uniform mat4 SM_ViewMatrixInv;
uniform float SM_ProjMatrix10;
uniform float SM_ProjMatrix14;
uniform int  SM_Perspective; // -1 for not use Shadow Map
uniform int  SM_CubicMap;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

in float t; 
in vec3 modelview_position;
in vec4 modelview_position4;
in vec3 normal;
in vec2 gsTexCoord;

//##@FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

//##FUNC_NORMAL##

//##FUNC_SHADOW_MAP##

void main (void)
{
    if (gl_FrontFacing == false) discard;
    
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture(textureImage, gsTexCoord.st);
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }

    //##APPLY_SMOOTHCLIPPING##

    if (t == 0){
	gl_FragColor = vec4(0.0, 0.0, 0.0, alpha);
	if (fogMode != 0) FogEffect();
	return;
    }
    
    vec3 fnormal = normalize(GetNormal());

    vec3  eye = - normalize(modelview_position);
    float eyedot = dot(eye, fnormal);
    if (eyedot * eyedot < edgeThreshold){
	gl_FragColor = vec4(0, 0, 0, alpha);
	if (fogMode != 0) FogEffect();
	//##APPLY_ENSEMBLE_DITHERING##
	return;
    }
    
    //##APPLY_ENSEMBLE_DITHERING##
    
    if (alpha > 1.0) alpha = 1.0;
    
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    vec3 viewVec = -normalize(modelview_position);
    
    for (int i = 0; i < lightNumber; i++){
	
	vec3 lightVec = gl_LightSource[i].position.xyz - modelview_position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }
    
    float s = diffuseTotal * 0.8 + specularTotal;
    //if (s > 1.0) s = 1.0;
    
    vec4 ambient = gl_FrontMaterial.ambient * float(shadowVolumeMode);

    // --- Use SHADOW Map --
    float colorFactor = 1;
    if (SM_Perspective != -1){
	//##APPLY_SHADOW_MAP##
    }
    
    if (s < 0.2){
	gl_FragColor = ambient;
    }
    else if (s < 0.75){
	gl_FragColor = ambient +
	    colorFactor * gl_FrontMaterial.diffuse;
    }
    else{
	gl_FragColor = ambient + 
	    colorFactor * (gl_FrontMaterial.diffuse + 
			   gl_FrontMaterial.specular);
    }
    
    gl_FragColor *= texColor;

    //##APPLY_FOG_EFFECT##
    
    gl_FragColor[3] = alpha;
}
