#version 130
#extension GL_EXT_gpu_shader4 : enable

uniform int lightNumber;
uniform int lightRecieve[7];

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

//uniform mat4 baseInvModelMatrix;//
//uniform mat4 modelInvBaseMatrix;//
uniform mat4 baseInvMatrix;
uniform mat4 rotationMatrix;

uniform int fogMode;
uniform int dissolveMode;

uniform sampler2D SM_Texture;
uniform mat4 SM_LightViewMatrix;
uniform float SM_OrthoSize; //OrthoScaleKv
uniform mat4 SM_ViewMatrixInv;
uniform float SM_ProjMatrix10;
uniform float SM_ProjMatrix14;
uniform int  SM_Perspective; // -1 for not use Shadow Map
uniform int  SM_CubicMap;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

in vec3 modelview_position;
in vec4 modelview_position4;
in vec4 color;
in vec4 colorDiffuse;
in vec4 colorSpecular;
in vec4 normal;
in vec4 tangent;
in vec4 binormal;
in vec2 gsTexCoord;
flat in int blend;
flat in int pseudoSphere;
flat in int textureImageIndex;

//##FUNC@_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SHADOW_MAP##

void main (void)
{
    
	

    // --- Use SHADOW Map --
    float colorFactor = 1;
    if (SM_Perspective != -1){
	//##APPLY_SHADOW_MAP##
    }

    vec4 texColor;
    vec4 clrBase = color;
    
    if (pseudoSphere == 0){
	clrBase = vec4(1, 1, 1, clrBase[3]);
    }

    if (pseudoSphere == 1)
	clrBase = vec4(1, 1, 1, clrBase[3]);
    
    
    vec2 texCoord;
    if (pseudoSphere == 1){
	float s = 2*(gsTexCoord.s-0.5);
	float t = 2*(gsTexCoord.t-0.5);
	float st2 = s*s+t*t;
	if (st2 >= 1) discard;

	vec4 pos = vec4(s, -sqrt(1-st2), t, 0);
	
	pos = baseInvMatrix * rotationMatrix * pos;

	float theta = -asin(pos[2]);
	float phi = -atan(pos[0], pos[1]);

	texCoord.s = phi / 3.14149 * 0.5 + 0.5 - 0.25;
	texCoord.t = theta / 3.14159 + 0.5;
    }
    else{
	texCoord = gsTexCoord.st;
    }

    if (textureImageIndex > 2){
	texColor = clrBase;
    }
    else{	
	if (textureSize(textureImage[textureImageIndex], 0).x > 1)
	    texColor = texture(textureImage[textureImageIndex], texCoord.st) * clrBase;
	else
	    texColor = clrBase;
    }
    
    float alpha = texColor[3];
    if (alpha == 0) discard;

    //##APPLY@_SMOOTHCLIPPING##

    //##APPLY_ENSEMBLE_DITHERING##

    if (pseudoSphere == 0){
	vec4 ambientClr = vec4(0, 0, 0, 0);
	vec3 diffuseTotal = vec3(0.0, 0.0, 0.0);
	
	for (int i = 0; i < lightNumber; i++){
	    if (lightRecieve[i] == 0) continue;
	    
	    ambientClr += gl_LightSource[i].ambient * color;

	    vec3 lightVec = gl_LightSource[i].position.xyz - modelview_position * gl_LightSource[i].position.w;
	    float dis = length(lightVec);

	    float diffuse = 1.00;
	    float lightFactor = 1.0f;
		
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);
		
	    diffuseTotal += diffuse * vec3(gl_LightSource[i].diffuse) * lightFactor;
	}

	texColor.rgb *= ambientClr.rgb + colorFactor * diffuseTotal*colorDiffuse.rgb;
    }


    if (pseudoSphere == 1){

	vec4 ambientClr = vec4(0, 0, 0, 0);
	
	vec3 diffuseTotal = vec3(0.0, 0.0, 0.0);
	vec3 specularTotal = vec3(0.0, 0.0, 0.0);
	
	float shininess = 20;
	float s = 2*(-gsTexCoord.s+0.5);
	float t = 2*(gsTexCoord.t-0.5);
	float st2 = s*s+t*t;
	
	vec3 normal_l = - sqrt(1-st2)*normal.xyz + s * tangent.xyz + t * binormal.xyz;//vec3(coordXY, 1 - length(coordXY));
	
	vec3 fnormal = normalize(normal_l);
	vec3 viewVec = -normalize(modelview_position);
	
	for (int i = 0; i < lightNumber; i++){
	    if (lightRecieve[i] == 0) continue;
	    
	    ambientClr += gl_LightSource[i].ambient * color;

	    vec3 lightVec = gl_LightSource[i].position.xyz - modelview_position * gl_LightSource[i].position.w;
	    float dis = length(lightVec);
	    lightVec = normalize(lightVec);
	    
	    float diffuse = dot(lightVec, fnormal);
	    
	    if (diffuse > 0.0){
		float lightFactor = 1.0f;
		
		lightFactor /= (gl_LightSource[i].constantAttenuation +
				gl_LightSource[i].linearAttenuation * dis +
				gl_LightSource[i].quadraticAttenuation * dis * dis);
		
		vec3 reflectVec = reflect(-lightVec, fnormal);
		float specular = 
		    pow(max(dot(viewVec, reflectVec), 0.0), shininess) * lightFactor;
		diffuseTotal += diffuse * vec3(gl_LightSource[i].diffuse) * lightFactor;
		specularTotal += specular * vec3(gl_LightSource[i].specular);
	    }
	}

	texColor.rgb *= ambientClr.rgb + colorFactor * (diffuseTotal*colorDiffuse.rgb + specularTotal*colorSpecular.rgb);

    }

    gl_FragColor = texColor;

    FogEffect();

    if (alpha > 1.0){
	gl_FragColor.rgb *= alpha;
	alpha = 1.0;
    }
    
    switch(blend){
    case 0:
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha;
	break;
    case 1:
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = 0;
	break;
    case 2:
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha * (1 - alpha);
	break;
    }
    
}
