// SVTriangleMesh.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_TRIANGLES_ADJACENCY_EXT
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLES
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=24

#version 130
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform float shadowLength;
uniform float offsetFactor;
uniform int   forClosed; //1 for Clesed, -1 for Opened

varying in vec3 positionIn[];
//varying in vec4 colorIn[];
//varying out vec4 color;

vec3 CastedS(const vec3 nlightVec, const vec3 pos){
    return pos + nlightVec * offsetFactor;
}

vec3 CastedL(const vec3 nlightVec, const vec3 pos){
    return pos + nlightVec ;
}

void main(void){

    vec3 lpos = gl_LightSource[0].position.xyz;
    float lw = gl_LightSource[0].position[3];

    vec3 ltVec0 = lpos - positionIn[0] * lw;
    
    vec3 polyNormal = cross(vec3(positionIn[2] - positionIn[0]), vec3(positionIn[4] - positionIn[0]));
    float litOrNot = dot(ltVec0, polyNormal) * forClosed;
    
    if (litOrNot > 0) return;

    vec3 ltVec2 = lpos - positionIn[2] * lw;
    vec3 ltVec4 = lpos - positionIn[4] * lw;
    
    vec3 polyNormalABdash = cross(vec3(positionIn[1] - positionIn[0]), vec3(positionIn[2] - positionIn[0]));
    float litOrNotABdash = dot(ltVec0, polyNormalABdash) * forClosed;

    vec3 polyNormalBCdash = cross(vec3(positionIn[3] - positionIn[2]), vec3(positionIn[4] - positionIn[2]));
    float litOrNotBCdash = dot(ltVec2, polyNormalBCdash) * forClosed;

    vec3 polyNormalCAdash = cross(vec3(positionIn[5] - positionIn[4]), vec3(positionIn[0] - positionIn[4]));
    float litOrNotCAdash = dot(ltVec4, polyNormalCAdash) * forClosed;

    vec3 nltVec0 = -normalize(ltVec0) * shadowLength;
    vec3 nltVec2 = -normalize(ltVec2) * shadowLength;
    vec3 nltVec4 = -normalize(ltVec4) * shadowLength;

    vec4 p0S = gl_Position = gl_ProjectionMatrix*vec4((positionIn[0]+ nltVec0*offsetFactor),1);
    EmitVertex();
    vec4 p2S = gl_Position = gl_ProjectionMatrix*vec4((positionIn[2]+ nltVec2*offsetFactor),1);
    EmitVertex();
    vec4 p4S = gl_Position = gl_ProjectionMatrix*vec4((positionIn[4]+ nltVec4*offsetFactor),1);
    EmitVertex();
    EndPrimitive();

    vec4 p0L = gl_Position = gl_ProjectionMatrix*vec4((positionIn[0]+ nltVec0), 1);
    EmitVertex();
    vec4 p4L = gl_Position = gl_ProjectionMatrix*vec4((positionIn[4]+ nltVec4), 1);
    EmitVertex();
    vec4 p2L = gl_Position = gl_ProjectionMatrix*vec4((positionIn[2]+ nltVec2), 1);
    EmitVertex();
    EndPrimitive();

    if (litOrNotABdash > 0){
	gl_Position = p0S; EmitVertex();
	gl_Position = p2L; EmitVertex();
	gl_Position = p2S; EmitVertex();
	EndPrimitive();
	
	gl_Position = p0S; EmitVertex();
	gl_Position = p0L; EmitVertex();
	gl_Position = p2L; EmitVertex();
	EndPrimitive();
    }

    if (litOrNotBCdash > 0){
	gl_Position = p2S; EmitVertex();
	gl_Position = p4L; EmitVertex();
	gl_Position = p4S; EmitVertex();
	EndPrimitive();
	
	gl_Position = p2S; EmitVertex();
	gl_Position = p2L; EmitVertex();
	gl_Position = p4L; EmitVertex();
	EndPrimitive();
    }

    if (litOrNotCAdash > 0){
	gl_Position = p4S; EmitVertex();
	gl_Position = p0L; EmitVertex();
	gl_Position = p0S; EmitVertex();
	EndPrimitive();
	
	gl_Position = p4S; EmitVertex();
	gl_Position = p4L; EmitVertex();
	gl_Position = p0L; EmitVertex();
	EndPrimitive();
    }

    
}
