#version 130
//#extension GL_EXT_gpu_shader4 : enable

uniform int lightNumber;

uniform int textureMode;
uniform sampler1D textureColorCurve;
uniform sampler2D textureSample;
uniform int fogMode;
uniform int dissolveMode;
uniform int useClip;
uniform vec3 clipBoxMin;
uniform vec3 clipBoxMax;
uniform mat4 matrix16;
uniform float smoothWidth;
uniform float thinningFactor;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

in vec4 texCoord;
in vec2 texCoord2;
in vec4 color;
in vec3 position;
in vec3 modelview_position;
in vec3 vertex;
flat in int lineIndex;
flat in float filterAlpha;

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

float clipFactor(float tmp){
    if (tmp > 0){
	if (tmp < smoothWidth) 
	    return 1.0f-tmp/smoothWidth * (1.0f-thinningFactor);
	else
	    return thinningFactor;
    }
    return 1;
}

void main(void)
{
    if (lineIndex == -1) discard;

    float alphaFactor = filterAlpha;
    if (useClip == 1){
	vec3 vertex2 = vec3(matrix16 * vec4(vertex, 1));
	if (smoothWidth <= 0 && thinningFactor == 0){
	    if (vertex2[0] < clipBoxMin[0]) discard;
	    if (vertex2[1] < clipBoxMin[1]) discard;
	    if (vertex2[2] < clipBoxMin[2]) discard;
	    if (vertex2[0] > clipBoxMax[0]) discard;
	    if (vertex2[1] > clipBoxMax[1]) discard;
	    if (vertex2[2] > clipBoxMax[2]) discard;
	}
	else{
	    float tmp; 
	    float alphaFactorAxis[3];
	    for (int xyz = 0; xyz < 3; xyz++) alphaFactorAxis[xyz] = 1;
	    
	    for (int xyz = 0; xyz < 3; xyz++){
		alphaFactorAxis[xyz] *= clipFactor(clipBoxMin[xyz]-vertex2[xyz]);
		alphaFactorAxis[xyz] *= clipFactor(vertex2[xyz]-clipBoxMax[xyz]);
	    }

	    for (int xyz = 0; xyz < 3; xyz++)
		if (alphaFactor > alphaFactorAxis[xyz]) 
		    alphaFactor = alphaFactorAxis[xyz];
	    if (alphaFactor == 0) discard;
	}
    }

    if (useClip == 2){
	vec3 vertex2 = vec3(matrix16 * vec4(vertex, 1));
	float radius = length(vertex2);
	if (smoothWidth <= 0 && thinningFactor == 0){
	    if (radius < clipBoxMin[0]) discard;
	    if (radius > clipBoxMax[0]) discard;
	}
	else{
	    alphaFactor *= clipFactor(clipBoxMin[0] - radius);
	    alphaFactor *= clipFactor(radius - clipBoxMax[0]);
	    if (alphaFactor == 0) discard;
	}
    }

    if (useClip == 3){
	vec3 vertex2 = vec3(matrix16 * vec4(vertex, 1));
	float radius = sqrt(vertex[0]*vertex[0] + vertex[1]*vertex[1]);
	if (smoothWidth <= 0 && thinningFactor == 0){
	    if (radius < clipBoxMin[0]) discard;
	    if (radius > clipBoxMax[0]) discard;
	    if (vertex2[2] < clipBoxMin[2]) discard;
	    if (vertex2[2] > clipBoxMax[2]) discard;
	}
	else{
	    float alphaFactorAxis[2];
	    alphaFactorAxis[0] = 1;
	    alphaFactorAxis[1] = 1;

	    alphaFactorAxis[0] *= clipFactor(clipBoxMin[0] - radius);
	    alphaFactorAxis[0] *= clipFactor(radius - clipBoxMax[0]);
	    alphaFactorAxis[1] *= clipFactor(clipBoxMin[2] - vertex2[2]);
	    alphaFactorAxis[1] *= clipFactor( vertex2[2] - clipBoxMax[2]);
	    
	    for (int i = 0; i < 2; i++)
		if (alphaFactor > alphaFactorAxis[i]) 
		    alphaFactor = alphaFactorAxis[i];
	    if (alphaFactor == 0) discard;
	}
    }
    
    vec4 texColor = texture(textureColorCurve, texCoord.s/*gl_TexCoord[0].s*/);
    if (textureMode == 0) texColor = vec4(1, 1, 1, 1);

    vec4 texColor2 = texture(textureSample, texCoord2.st);

    texColor = texColor * texColor2;
    if (texColor[3] > 1) texColor[3] = 1;
    
    float alpha = texColor[3] * alphaFactor;
    if (alpha == 0) discard;

    //##APPLY_ENSEMBLE_DITHERING##
    
    gl_FragColor.r = color.r * texColor.r;
    gl_FragColor.g = color.g * texColor.g;
    gl_FragColor.b = color.b * texColor.b;

    FogEffect();
    
    alpha *= color.a;

    int blend = 0;
    if (blend == 0){
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha;
    }
    else{
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = 0;
    }
}
